@@ -24,6 +24,7 @@
 #include "mnb-drop-down.h"
 #include "mnb-toolbar.h"    /* For MNB_IS_TOOLBAR */
 #include "moblin-netbook.h" /* For PANEL_HEIGHT */
+#include "switcher/mnb-switcher.h"
 
 #define SLIDE_DURATION 150
 
@@ -158,6 +159,92 @@ mnb_toolbar_show_completed_cb (MnbToolbar *toolbar, gpointer data)
   clutter_actor_show (dropdown);
 }
 
+/*
+ * Ensure that the drop down does not stretch over any reserved strut at the
+ * bottom of the screen (e.g., if VKB is present)
+ */
+void
+mnb_drop_down_ensure_size (MnbDropDown *self)
+{
+  MnbDropDownPrivate *priv  = MNB_DROP_DOWN (self)->priv;
+  ClutterActor       *actor = CLUTTER_ACTOR (self);
+
+  if (priv->child)
+    {
+      MetaRectangle  r;
+      MetaScreen    *screen;
+      MetaWorkspace *workspace;
+
+      screen    = mutter_plugin_get_screen (priv->plugin);
+      workspace = meta_screen_get_active_workspace (screen);
+
+      if (workspace)
+        {
+          gfloat x, y, w, h, wc, hc;
+          gint   xi, yi, wi, hi, wci, hci;
+          gint   max_height, max_inner_height, inner_width;
+
+          meta_workspace_get_work_area_all_monitors (workspace, &r);
+
+          clutter_actor_get_position (actor, &x, &y);
+          clutter_actor_get_size (actor, &w, &h);
+          clutter_actor_get_size (priv->child, &wc, &hc);
+
+          xi  = (gint)x;
+          yi  = (gint)y;
+          wi  = (gint)w;
+          hi  = (gint)h;
+          wci = (gint)wc;
+          hci = (gint)hc;
+
+          /*
+           * Maximum height of the panel is the available working height plus the
+           * height of the panel shadow (we allow the shaddow to stretch out of
+           * the available area).
+           */
+          max_height = r.y + r.height - yi + MNB_DROP_DOWN_SHADOW_HEIGHT;
+
+          /*
+           * inner height is height of the dropdown child, i.e., the max height
+           * minus the height of the shadow, minus the height of the footer
+           * (half toolbar height), minus the y padding in the panel.
+           */
+          max_inner_height = max_height - MNB_DROP_DOWN_SHADOW_HEIGHT -
+            TOOLBAR_HEIGHT/2 - 4;
+
+          inner_width = r.width - TOOLBAR_X_PADDING * 2;
+
+          /*
+           * We have to test the size of the child here, as the external size
+           * might not be correct (e.g., when initially showing the Switcher the
+           * external size matches the size of the child, i.e., the styling is
+           * not yet applied.
+           *
+           * When initially showing the OOP panels, the child is not yet
+           * allocated, and will have size 0; in that case we force the resize
+           * otherwise the child would get allocated with the size specified at
+           * construction.
+           *
+           * We rest the size whenever the max height does not match that of the
+           * child, so that the window gets resized in both up and down (the
+           * height passed into mpl_panel_set_height() is really a maximum
+           * allowable height, and if the panel client asked for height lesser
+           * than this, the original height request will be respected.
+           */
+          if (max_inner_height != hci || inner_width != wci)
+            {
+              if (MNB_IS_PANEL (actor))
+                mnb_panel_set_size ((MnbPanel*)actor, r.width, max_height);
+              else if (MNB_IS_SWITCHER (actor))
+                mnb_switcher_set_size ((MnbSwitcher*)actor,
+                                       r.width, max_height);
+              else
+                clutter_actor_set_size (actor, w, (gfloat) max_height);
+            }
+        }
+    }
+}
+
 static void
 mnb_drop_down_show (ClutterActor *actor)
 {
@@ -175,15 +262,16 @@ mnb_drop_down_show (ClutterActor *actor)
 
   if (priv->hide_completed_id)
     {
-      g_debug ("disconnecting hide_completed_cb");
       g_signal_handler_disconnect (priv->hide_anim, priv->hide_completed_id);
       priv->hide_anim = NULL;
       priv->hide_completed_id = 0;
       priv->in_hide_animation = FALSE;
     }
 
+  mnb_drop_down_ensure_size ((MnbDropDown*)actor);
+
   /*
-   * Check the panel is visible, if not show it.
+   * Check the toolbar is visible, if not show it.
    */
   toolbar = clutter_actor_get_parent (actor);
   while (toolbar && !MNB_IS_TOOLBAR (toolbar))
@@ -266,7 +354,7 @@ mnb_drop_down_hide_completed_cb (ClutterAnimation *anim, ClutterActor *actor)
         toolbar = clutter_actor_get_parent (toolbar);
 
       if (toolbar)
-        clutter_actor_hide (toolbar);
+        mnb_toolbar_hide ((MnbToolbar*)toolbar);
 
       priv->hide_toolbar = FALSE;
     }
@@ -292,7 +380,6 @@ mnb_drop_down_hide (ClutterActor *actor)
 
   if (priv->show_completed_id)
     {
-      g_debug ("disconnecting show_completed_cb");
       g_signal_handler_disconnect (priv->show_anim, priv->show_completed_id);
       priv->show_anim = NULL;
       priv->show_completed_id = 0;
@@ -392,7 +479,9 @@ mnb_drop_down_allocate (ClutterActor          *actor,
                         const ClutterActorBox *box,
                         ClutterAllocationFlags flags)
 {
+#if 0
   MnbDropDownPrivate *priv = MNB_DROP_DOWN (actor)->priv;
+#endif
   ClutterActorClass  *parent_class;
 
   /*
@@ -433,6 +522,22 @@ mnb_drop_down_allocate (ClutterActor          *actor,
   parent_class->allocate (actor, box, flags);
 }
 
+static void
+mnb_drop_down_constructed (GObject *object)
+{
+  MnbDropDownPrivate *priv = MNB_DROP_DOWN (object)->priv;
+  NbtkWidget         *footer;
+
+  /* footer with "up" button */
+  footer = nbtk_button_new ();
+  nbtk_widget_set_style_class_name (footer, "drop-down-footer");
+  nbtk_table_add_actor (NBTK_TABLE (object), CLUTTER_ACTOR (footer), 1, 0);
+  g_signal_connect_swapped (footer, "clicked",
+                            G_CALLBACK (mnb_drop_down_hide_with_toolbar), object);
+
+  priv->footer = CLUTTER_ACTOR (footer);
+}
+
 static void
 mnb_drop_down_class_init (MnbDropDownClass *klass)
 {
@@ -445,6 +550,7 @@ mnb_drop_down_class_init (MnbDropDownClass *klass)
   object_class->set_property = mnb_drop_down_set_property;
   object_class->dispose = mnb_drop_down_dispose;
   object_class->finalize = mnb_drop_down_finalize;
+  object_class->constructed = mnb_drop_down_constructed;
 
   clutter_class->show = mnb_drop_down_show;
   clutter_class->hide = mnb_drop_down_hide;
@@ -503,24 +609,12 @@ mnb_drop_down_class_init (MnbDropDownClass *klass)
 static void
 mnb_drop_down_init (MnbDropDown *self)
 {
-  NbtkWidget *footer;
-  MnbDropDownPrivate *priv;
-
-  priv = self->priv = GET_PRIVATE (self);
-
-  /* footer with "up" button */
-  footer = nbtk_button_new ();
-  nbtk_widget_set_style_class_name (footer, "drop-down-footer");
-  nbtk_table_add_actor (NBTK_TABLE (self), CLUTTER_ACTOR (footer), 1, 0);
-  g_signal_connect_swapped (footer, "clicked",
-                            G_CALLBACK (mnb_drop_down_hide_with_toolbar), self);
+  self->priv = GET_PRIVATE (self);
 
   g_object_set (self,
                 "show-on-set-parent", FALSE,
                 "reactive", TRUE,
                 NULL);
-
-  priv->footer = CLUTTER_ACTOR (footer);
 }
 
 NbtkWidget*
@@ -570,20 +664,19 @@ mnb_drop_down_set_child (MnbDropDown *drop_down,
         }
 
       clutter_container_remove_actor (CLUTTER_CONTAINER (drop_down),
-                                      drop_down->priv->child);
+                                      priv->child);
     }
 
   if (child)
     {
-      drop_down->priv->reparent_cb =
+      priv->reparent_cb =
         g_signal_connect (child, "parent-set",
                           G_CALLBACK (mnb_drop_down_reparent_cb),
                           drop_down);
       nbtk_table_add_actor (NBTK_TABLE (drop_down), child, 0, 0);
     }
 
-  drop_down->priv->child = child;
-
+  priv->child = child;
 }
 
 ClutterActor*
