@@ -28,6 +28,7 @@
 #include "mnb-panel.h"
 #include "marshal.h"
 
+#include <string.h>
 #include <dbus/dbus-glib.h>
 #include <dbus/dbus-glib-bindings.h>
 #include <dbus/dbus-glib-lowlevel.h>
@@ -36,6 +37,16 @@
 #include <gdk/gdkx.h>
 #include <moblin-panel/mpl-panel-common.h>
 #include <display.h>
+#include <errors.h>
+
+/*
+ * Including mutter's errors.h defines the i18n macros, so undefine them before
+ * including the glib i18n header.
+ */
+#undef _
+#undef N_
+
+#include <X11/Xatom.h>
 
 G_DEFINE_TYPE (MnbPanel, mnb_panel, MNB_TYPE_DROP_DOWN)
 
@@ -84,6 +95,7 @@ struct _MnbPanelPrivate
   gchar           *button_style_id;
   guint            xid;
   guint            child_xid;
+  gchar           *child_class;
 
   GtkWidget       *window;
 
@@ -148,24 +160,29 @@ mnb_panel_set_property (GObject *object, guint property_id,
 void
 mnb_panel_focus (MnbPanel *panel)
 {
-  MetaDisplay *display;
-  MutterPlugin *plugin = NULL;
+  MnbPanelPrivate *priv = panel->priv;
+  MetaDisplay     *display;
+  MutterPlugin    *plugin = moblin_netbook_get_plugin_singleton ();
+  guint32          timestamp;
 
-  g_object_get (G_OBJECT (panel), "mutter-plugin", &plugin, NULL);
-  if (!plugin)
-        return;
+  display   = meta_screen_get_display (mutter_plugin_get_screen (plugin));
+  timestamp = meta_display_get_current_time_roundtrip (display);
 
-  display = meta_screen_get_display (mutter_plugin_get_screen (plugin));
+  meta_error_trap_push (display);
 
-  gdk_error_trap_push ();
+  XRaiseWindow (meta_display_get_xdisplay (display), priv->xid);
+  XSync (meta_display_get_xdisplay (display), False);
 
-  XRaiseWindow (meta_display_get_xdisplay (display), panel->priv->xid);
+  if (priv->mcw)
+    meta_display_set_input_focus_window (display,
+                                     mutter_window_get_meta_window (priv->mcw),
+                                     FALSE,
+                                     timestamp);
 
-  XSetInputFocus (meta_display_get_xdisplay (display), panel->priv->child_xid,
-                  RevertToPointerRoot, CurrentTime);
+  XSetInputFocus (meta_display_get_xdisplay (display), priv->child_xid,
+                  RevertToPointerRoot, timestamp);
 
-  gdk_flush ();
-  gdk_error_trap_pop ();
+  meta_error_trap_pop (display, TRUE);
 }
 
 /*
@@ -316,6 +333,7 @@ mnb_panel_finalize (GObject *object)
   g_free (priv->tooltip);
   g_free (priv->stylesheet);
   g_free (priv->button_style_id);
+  g_free (priv->child_class);
 
   G_OBJECT_CLASS (mnb_panel_parent_class)->finalize (object);
 }
@@ -488,7 +506,8 @@ mnb_panel_show_completed (MnbDropDown *self)
   clutter_actor_get_position (CLUTTER_ACTOR (self), &x, &y);
 
   gtk_window_move (GTK_WINDOW (priv->window),
-                   (gint)x + TOOLBAR_X_PADDING, (gint)y);
+                   (gint)x + TOOLBAR_X_PADDING,
+                   (gint)y + MNB_DROP_DOWN_TOP_PADDING);
 
   mnb_panel_focus (MNB_PANEL (self));
 
@@ -658,6 +677,56 @@ mnb_panel_init_panel_reply_cb (DBusGProxy *proxy,
 
   priv->child_xid = xid;
 
+  g_free (priv->child_class);
+
+  /*
+   * Retrieve the WM_CLASS property for the child window (we have to do it the
+   * hard way, because the WM_CLASS on the MutterWindow is coming from mutter,
+   * not the application).
+   *
+   * (We use the wm-class to identify sub-windows.)
+   */
+  {
+    Atom r_type;
+    int  r_fmt;
+    unsigned long n_items;
+    unsigned long r_after;
+    char *r_prop;
+    MutterPlugin *plugin = moblin_netbook_get_plugin_singleton ();
+    MetaDisplay *display;
+
+    display = meta_screen_get_display (mutter_plugin_get_screen (plugin));
+
+    meta_error_trap_push (display);
+
+    if (Success == XGetWindowProperty (GDK_DISPLAY (), xid, XA_WM_CLASS,
+                                       0, 8192,
+                                       False, XA_STRING,
+                                       &r_type, &r_fmt, &n_items, &r_after,
+                                       (unsigned char **)&r_prop) &&
+        r_type != 0)
+      {
+        if (r_prop)
+          {
+            /*
+             * The property contains two strings separated by \0; we want the
+             * second string.
+             */
+            gint len0 = strlen (r_prop);
+
+            if (len0 == n_items)
+              len0--;
+
+            priv->child_class = g_strdup (r_prop + len0 + 1);
+
+            g_debug ("child class %s", priv->child_class);
+            XFree (r_prop);
+          }
+      }
+
+    meta_error_trap_pop (display, TRUE);
+  }
+
   /*
    * If we already have a window, we are being called because the panel has
    * died on us; we simply destroy the window and start again. (It should be
@@ -739,21 +808,15 @@ mnb_panel_proxy_owner_changed_cb (DBusGProxy *proxy,
   mnb_panel_init_owner (MNB_PANEL (data));
 }
 
-static gboolean
-mnb_panel_setup_proxy (MnbPanel *panel)
+DBusGProxy *
+mnb_panel_create_dbus_proxy (DBusGConnection *dbus_conn,
+                             const gchar     *dbus_name)
 {
-  MnbPanelPrivate *priv = panel->priv;
-  DBusGProxy      *proxy;
-  gchar           *dbus_path;
-  gchar           *p;
+  DBusGProxy *proxy;
+  gchar      *dbus_path;
+  gchar      *p;
 
-  if (!priv->dbus_conn)
-    {
-      g_warning (G_STRLOC " No dbus connection, cannot connect to panel!");
-      return FALSE;
-    }
-
-  dbus_path = g_strconcat ("/", priv->dbus_name, NULL);
+  dbus_path = g_strconcat ("/", dbus_name, NULL);
 
   p = dbus_path;
   while (*p)
@@ -764,13 +827,30 @@ mnb_panel_setup_proxy (MnbPanel *panel)
       ++p;
     }
 
-  proxy = dbus_g_proxy_new_for_name (priv->dbus_conn,
-                                     priv->dbus_name,
+  proxy = dbus_g_proxy_new_for_name (dbus_conn,
+                                     dbus_name,
                                      dbus_path,
                                      MPL_PANEL_DBUS_INTERFACE);
 
   g_free (dbus_path);
 
+  return proxy;
+}
+
+static gboolean
+mnb_panel_setup_proxy (MnbPanel *panel)
+{
+  MnbPanelPrivate *priv = panel->priv;
+  DBusGProxy      *proxy;
+
+  if (!priv->dbus_conn)
+    {
+      g_warning (G_STRLOC " No dbus connection, cannot connect to panel!");
+      return FALSE;
+    }
+
+  proxy = mnb_panel_create_dbus_proxy (priv->dbus_conn, priv->dbus_name);
+
   if (!proxy)
     {
       g_warning ("Unable to create proxy for %s (reason unknown)",
@@ -1070,3 +1150,80 @@ mnb_panel_get_mutter_window (MnbPanel *panel)
   return priv->mcw;
 }
 
+/*
+ * Retruns TRUE if the passed window is both distinct from the panel window,
+ * and belongs to the same window class.
+ */
+gboolean
+mnb_panel_owns_window (MnbPanel *panel, MutterWindow *mcw)
+{
+  MnbPanelPrivate *priv = panel->priv;
+  const gchar     *wclass;
+
+  if (!priv->mcw || !mcw)
+    return FALSE;
+
+  /*
+   * Return FALSE for the top level panel window.
+   */
+  if (priv->mcw == mcw)
+    return FALSE;
+
+  wclass = meta_window_get_wm_class (mutter_window_get_meta_window (mcw));
+
+  if (priv->child_class && wclass && !strcmp (priv->child_class, wclass))
+    return TRUE;
+
+  return FALSE;
+}
+
+/*
+ * Returns TRUE if the passed window is both distinct from the panel window,
+ * and transient for it.
+ */
+gboolean
+mnb_panel_is_ancestor_of_transient (MnbPanel *panel, MutterWindow *mcw)
+{
+  MutterWindow *pcw;
+  MetaWindow   *pmw, *mw;
+
+  if (!panel)
+    return FALSE;
+
+  pcw = mnb_panel_get_mutter_window (panel);
+
+  if (!pcw || pcw == mcw)
+    return FALSE;
+
+  pmw = mutter_window_get_meta_window (pcw);
+  mw  = mutter_window_get_meta_window (mcw);
+
+  return meta_window_is_ancestor_of_transient (pmw, mw);
+}
+
+static void
+mnb_panel_dbus_ping_cb (DBusGProxy *proxy, GError *error, gpointer data)
+{
+  g_debug ("Ping to %s acknowledged", (gchar*)data);
+  g_free (data);
+}
+
+
+void
+mnb_toolbar_ping_panel (DBusGConnection *dbus_conn, const gchar *dbus_name)
+{
+  DBusGProxy *proxy = mnb_panel_create_dbus_proxy (dbus_conn, dbus_name);
+
+  if (!proxy)
+    {
+      g_warning ("Unable to create proxy for %s (reason unknown)", dbus_name);
+      return;
+    }
+
+  org_moblin_UX_Shell_Panel_ping_async (proxy,
+                                        mnb_panel_dbus_ping_cb,
+                                        g_strdup (dbus_name));
+
+  g_object_unref (proxy);
+}
+
