@@ -61,7 +61,8 @@
 #define TOOLBAR_TRIGGER_THRESHOLD       1
 #define TOOLBAR_TRIGGER_THRESHOLD_TIMEOUT 500
 #define TOOLBAR_LOWLIGHT_FADE_DURATION 300
-#define TOOLBAR_AUTOSTART_DELAY 10
+#define TOOLBAR_AUTOSTART_DELAY 15
+#define TOOLBAR_AUTOSTART_ATTEMPTS 10
 #define TOOLBAR_WAITING_FOR_PANEL_TIMEOUT 1 /* in seconds */
 #define MOBLIN_BOOT_COUNT_KEY "/desktop/moblin/myzone/boot_count"
 
@@ -1772,8 +1773,11 @@ mnb_toolbar_noc_cb (DBusGProxy  *proxy,
 static gboolean
 mnb_toolbar_autostart_panels_cb (gpointer toolbar)
 {
+  static gint count = 0;
+
   MnbToolbarPrivate  *priv = MNB_TOOLBAR (toolbar)->priv;
   gint                i;
+  gboolean            missing = FALSE;
 
   for (i = 0; i < NUM_ZONES; ++i)
     {
@@ -1787,6 +1791,12 @@ mnb_toolbar_autostart_panels_cb (gpointer toolbar)
         case MYZONE:
         case MEDIA_ZONE:
         case STATUS_ZONE:
+        case WIFI_APPLET:
+        case VOLUME_APPLET:
+#if 0
+        case BATTERY_APPLET:
+        case BT_APPLET:
+#endif
           if (!priv->panels[i])
             {
               DBusConnection *conn;
@@ -1804,13 +1814,28 @@ mnb_toolbar_autostart_panels_cb (gpointer toolbar)
               dbus_bus_start_service_by_name (conn, dbus_name, 0, NULL, NULL);
 
               g_free (dbus_name);
+
+              missing = TRUE;
+
+                if (count > TOOLBAR_AUTOSTART_ATTEMPTS)
+                  {
+                    g_warning ("Panel %s is still not running after %d "
+                               "attempts to start it, last attempt.",
+                               mnb_toolbar_panel_index_to_name (i),
+                               count);
+                  }
             }
           break;
         default:;
         }
     }
 
-  return FALSE;
+  if (!missing || count > TOOLBAR_AUTOSTART_ATTEMPTS)
+    return FALSE;
+
+  count++;
+
+  return TRUE;
 }
 
 /*
@@ -2648,28 +2673,6 @@ mnb_toolbar_is_waiting_for_panel (MnbToolbar *toolbar)
   return (priv->waiting_for_panel_show || priv->waiting_for_panel_hide);
 }
 
-gboolean
-mnb_toolbar_window_is_transient_for_active_panel (MnbToolbar   *toolbar,
-                                                  MutterWindow *mcw)
-{
-  MnbPanel     *panel = (MnbPanel*)mnb_toolbar_get_active_panel (toolbar);
-  MutterWindow *pcw;
-  MetaWindow   *pmw, *mw;
-
-  if (!panel)
-    return FALSE;
-
-  pcw = mnb_panel_get_mutter_window (panel);
-
-  if (!pcw)
-    return FALSE;
-
-  pmw = mutter_window_get_meta_window (pcw);
-  mw  = mutter_window_get_meta_window (mcw);
-
-  return meta_window_is_ancestor_of_transient (pmw, mw);
-}
-
 void
 mnb_toolbar_foreach_panel (MnbToolbar        *toolbar,
                            MnbToolbarCallback callback,
@@ -2682,3 +2685,21 @@ mnb_toolbar_foreach_panel (MnbToolbar        *toolbar,
     if (priv->panels[i])
       callback ((MnbDropDown*)priv->panels[i], data);
 }
+
+gboolean
+mnb_toolbar_owns_window (MnbToolbar *toolbar, MutterWindow *mcw)
+{
+  MnbToolbarPrivate *priv  = toolbar->priv;
+  gint               i;
+
+  if (!mcw)
+    return FALSE;
+
+  for (i = 0; i < NUM_ZONES; i++)
+    if (priv->panels[i] && MNB_IS_PANEL (priv->panels[i]))
+      if (mnb_panel_owns_window ((MnbPanel*)priv->panels[i], mcw))
+        return TRUE;
+
+  return FALSE;
+}
+
