@@ -100,15 +100,14 @@ void mnb_toolbar_set_panel_input_only (MnbToolbar *toolbar, gboolean whether);
 
 gboolean mnb_toolbar_is_waiting_for_panel (MnbToolbar *toolbar);
 
-gboolean mnb_toolbar_window_is_transient_for_active_panel (MnbToolbar *toolbar,
-                                                           MutterWindow *mcw);
-
 void mnb_toolbar_hide (MnbToolbar *toolbar);
 
 void mnb_toolbar_foreach_panel (MnbToolbar        *toolbar,
                                 MnbToolbarCallback callback,
                                 gpointer           data);
 
+gboolean mnb_toolbar_owns_window (MnbToolbar *toolbar, MutterWindow *mcw);
+
 G_END_DECLS
 
 #endif /* _MNB_TOOLBAR */
