@@ -25,15 +25,16 @@
 #ifndef MOBLIN_NETBOOK_H
 #define MOBLIN_NETBOOK_H
 
+#include <gconf/gconf-client.h>
 #include <mutter-plugin.h>
 #include <nbtk/nbtk.h>
 
 #define SN_API_NOT_YET_FROZEN 1
 #include <libsn/sn.h>
 
-#include "moblin-netbook-notify-store.h"
-#include "mnb-notification-cluster.h"
-#include "mnb-notification-urgent.h"
+#include "notifications/moblin-netbook-notify-store.h"
+#include "notifications/mnb-notification-cluster.h"
+#include "notifications/mnb-notification-urgent.h"
 
 #include "mnb-input-manager.h"
 
@@ -85,9 +86,10 @@ struct _MoblinNetbookPluginPrivate
 
   MetaWindow            *last_focused;
 
-  gint                   fullscreen_apps [MAX_WORKSPACES + 1];
+  GList                 *fullscreen_wins;
 
-  gboolean               holding_focus : 1;
+  gboolean               holding_focus       : 1;
+  gboolean               compositor_disabled : 1;
 
   /* Background desktop texture */
   ClutterActor          *desktop_tex;
@@ -101,6 +103,11 @@ struct _MoblinNetbookPluginPrivate
   MnbInputRegion        *notification_urgent_input_region;
 
   Window                 focus_xwin;
+
+  /* Desktop background stuff */
+  GConfClient           *gconf_client;
+
+  MoblinNetbookNotifyStore *notify_store;
 };
 
 GType moblin_netbook_plugin_get_type (void);
@@ -153,4 +160,10 @@ moblin_netbook_get_plugin_singleton (void);
 gboolean
 moblin_netbook_modal_windows_present (MutterPlugin *plugin, gint workspace);
 
+gboolean
+moblin_netbook_compositor_disabled (MutterPlugin *plugin);
+
+void
+moblin_netbook_activate_window (MetaWindow *window);
+
 #endif
